@@ -1,5 +1,5 @@
 (function() {var implementors = {};
-implementors['std'] = [];implementors['collections'] = [];implementors['core'] = [];implementors['rustc_unicode'] = [];
+implementors['std'] = [];implementors['collections'] = [];implementors['core'] = [];implementors['rustc_unicode'] = [];implementors['std'] = [];implementors['collections'] = [];implementors['core'] = [];implementors['rustc_unicode'] = [];
 
             if (window.register_implementors) {
                 window.register_implementors(implementors);
