@@ -1,5 +1,5 @@
 (function() {var implementors = {};
-implementors['std'] = ["impl <a class='trait' href='rand/trait.Rng.html' title='rand::Rng'>Rng</a> for ThreadRng",];
+implementors['std'] = ["impl <a class='trait' href='rand/trait.Rng.html' title='rand::Rng'>Rng</a> for ThreadRng",];implementors['std'] = ["impl <a class='trait' href='rand/trait.Rng.html' title='rand::Rng'>Rng</a> for ThreadRng",];
 
             if (window.register_implementors) {
                 window.register_implementors(implementors);
